==> remote-booter-2014.7.6-sources.jar: com/xebialabs/deployit/booter/remote/xml/MethodDescriptorConverter.java <==
package com.xebialabs.deployit.booter.remote.xml;

import java.util.HashMap;
import java.util.Map;

import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

import com.xebialabs.deployit.booter.remote.DeployitCommunicator;
import com.xebialabs.deployit.booter.remote.RemoteBooter;
import com.xebialabs.deployit.booter.remote.RemoteMethodDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.MethodDescriptor;
import com.xebialabs.xltype.serialization.xstream.XStreamProvider;

@XStreamProvider(tagName = "control-task", readable = MethodDescriptor.class)
public class MethodDescriptorConverter implements Converter {

    @Override
    public void marshal(Object source, HierarchicalStreamWriter writer, MarshallingContext context) {
        throw new IllegalStateException("Cannot serialize MethodDescriptor from remote-booter");
    }

    @Override
    public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
        String booterConfigKey = (String) context.get("BOOTER_CONFIG");
        DeployitCommunicator communicator = RemoteBooter.getCommunicator(booterConfigKey);
        RemoteMethodDescriptor methodDescriptor = new RemoteMethodDescriptor();
        readXmlAttributes(reader, methodDescriptor, communicator);

        while (reader.hasMoreChildren()) {
            reader.moveDown();
            if (reader.getNodeName().equals("attributes")) {
                methodDescriptor.setAttributes(readAttributes(reader));
            }
            reader.moveUp();
        }

        return methodDescriptor;
    }

    private void readXmlAttributes(final HierarchicalStreamReader reader, final RemoteMethodDescriptor methodDescriptor, DeployitCommunicator communicator) {
        methodDescriptor.setName(reader.getAttribute("name"));
        methodDescriptor.setFqn(reader.getAttribute("fqn"));
        methodDescriptor.setDescription(reader.getAttribute("description"));
        methodDescriptor.setLabel(reader.getAttribute("label"));
        String attribute = reader.getAttribute("parameters-type");
        if (attribute != null && attribute.trim().length() > 0) {
            methodDescriptor.setParameterObjectType(communicator.getType(attribute));
        }
    }

    private Map<String, String> readAttributes(final HierarchicalStreamReader reader) {
        Map<String, String> map = new HashMap<String, String>();
        while (reader.hasMoreChildren()) {
            reader.moveDown();
            map.put(reader.getNodeName(), reader.getValue());
            reader.moveUp();
        }
        return map;
    }

    @Override
    public boolean canConvert(Class type) {
        return MethodDescriptor.class.equals(type);
    }
}

==> remote-booter-2015.2.2-sources.jar: com/xebialabs/deployit/booter/remote/xml/MethodDescriptorConverter.java <==
/**
 * Copyright © 2014-2015 XebiaLabs B.V. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.deployit.booter.remote.xml;

import java.util.HashMap;
import java.util.Map;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

import com.xebialabs.deployit.booter.remote.DeployitCommunicator;
import com.xebialabs.deployit.booter.remote.RemoteBooter;
import com.xebialabs.deployit.booter.remote.RemoteMethodDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.MethodDescriptor;
import com.xebialabs.xltype.serialization.xstream.XStreamProvider;

@XStreamProvider(tagName = "control-task", readable = MethodDescriptor.class)
public class MethodDescriptorConverter implements Converter {

    @Override
    public void marshal(Object source, HierarchicalStreamWriter writer, MarshallingContext context) {
        throw new IllegalStateException("Cannot serialize MethodDescriptor from remote-booter");
    }

    @Override
    public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
        String booterConfigKey = (String) context.get("BOOTER_CONFIG");
        DeployitCommunicator communicator = RemoteBooter.getCommunicator(booterConfigKey);
        RemoteMethodDescriptor methodDescriptor = new RemoteMethodDescriptor();
        readXmlAttributes(reader, methodDescriptor, communicator);

        while (reader.hasMoreChildren()) {
            reader.moveDown();
            if (reader.getNodeName().equals("attributes")) {
                methodDescriptor.setAttributes(readAttributes(reader));
            }
            reader.moveUp();
        }

        return methodDescriptor;
    }

    private void readXmlAttributes(final HierarchicalStreamReader reader, final RemoteMethodDescriptor methodDescriptor, DeployitCommunicator communicator) {
        methodDescriptor.setName(reader.getAttribute("name"));
        methodDescriptor.setFqn(reader.getAttribute("fqn"));
        methodDescriptor.setDescription(reader.getAttribute("description"));
        methodDescriptor.setLabel(reader.getAttribute("label"));
        String attribute = reader.getAttribute("parameters-type");
        if (attribute != null && !attribute.trim().isEmpty()) {
            methodDescriptor.setParameterObjectType(communicator.getType(attribute));
        }
    }

    private Map<String, String> readAttributes(final HierarchicalStreamReader reader) {
        Map<String, String> map = new HashMap<>();
        while (reader.hasMoreChildren()) {
            reader.moveDown();
            map.put(reader.getNodeName(), reader.getValue());
            reader.moveUp();
        }
        return map;
    }

    @Override
    public boolean canConvert(Class type) {
        return MethodDescriptor.class.equals(type);
    }
}
